import { License } from '../integration/index.js';
import { Request, UrlTools, Crypt } from '../util/index.js';
import { AssistantInstaller, DockerAssistant } from './index.js';
import { Login as LoginProvider } from '../provider/index.js';
import { DBConnector } from '../db/index.js';
class Login {
    constructor() {
        this.defaultProcotol = 'http';
    }
    isValidUser(user) {
        if (!user || !user.ID)
            throw new Error('INVALID_USER');
    }
    async getToken(serverHost, { CDOPERADOR, NMOPERADOR, NRORG }) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${NRORG}/API/generateToken`;
        const response = await Request.retryCall(route, {
            method: 'POST',
            data: { CDOPERADOR, NMOPERADOR },
        });
        return response.token;
    }
    async testDbConnection(serverHost, orgId) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${orgId}/API/dbConnection`;
        const response = await Request.retryCall(route, { method: 'GET' });
        return response;
    }
    async getAccessProfile(serverHost, { CDOPERADOR, NRORG }) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${NRORG}/API/accessProfile`;
        const accessProfile = await Request.post(route, { CDOPERADOR });
        return accessProfile;
    }
    async getOracleConnection() {
        const driver = DBConnector.connect('ORACLE');
        const { DB_SAAS_USER = '', DB_SAAS_PASSWORD = '', DB_SAAS_HOST = '', DB_SAAS_PORT = '', DB_SAAS_DBNAME = '', DB_SAAS_USE_WALLET = '', } = process.env;
        const useWallet = DB_SAAS_USE_WALLET === 'YES';
        const params = {
            dbName: DB_SAAS_DBNAME,
            useWallet
        };
        if (!useWallet) {
            params.host = DB_SAAS_HOST;
            params.user = DB_SAAS_USER;
            params.password = DB_SAAS_PASSWORD;
            params.port = DB_SAAS_PORT;
        }
        return driver.connect(params);
    }
    async getUserInLicense({ email, password }) {
        try {
            const response = await License.auth({ email, password });
            if (!response.dataset) {
                throw new Error('ERR_CALL_LICENSE');
            }
            const { dataset: { operador } } = response;
            return Array.isArray(operador) ? operador[0] : operador;
        }
        catch (err) {
            throw new Error('ERR_CALL_LICENSE');
        }
    }
    async getUserByAuth({ email, password }) {
        try {
            const conn = await this.getOracleConnection();
            const { rows: [operator] } = await LoginProvider.getUserByAuth(conn, { USER_OR_EMAIL: email, PASSWORD: Crypt.md5(password) });
            if (!operator) {
                const licenseOperator = await this.getUserInLicense({ email, password });
                if (licenseOperator) {
                    return licenseOperator;
                }
            }
            return operator;
        }
        catch (err) {
            console.error(err.message);
        }
    }
    async setServerForOrganization(params) {
        return License.setServerForOrganization(params);
    }
    async login({ email, password, agent, agentVersion }) {
        let errorInfo = {};
        const defaultErrorHandler = (e, errorCode) => {
            errorInfo = {
                message: e.message,
                stack: e.stack,
            };
            throw new Error(errorCode);
        };
        try {
            const operador = await this.getUserByAuth({ email, password });
            if (!operador) {
                throw new Error('INVALID_USER_OR_PASSWORD');
            }
            const { NRORG, ID: CDOPERADOR, NAME: NMOPERADOR, IS_ACTIVE = 'N' } = operador;
            if (NRORG && NRORG.toString() === '20') {
                throw new Error('ERR_UNAVAILABLE_ORG_20');
            }
            if (IS_ACTIVE === 'N') {
                throw new Error('ERR_OPERATOR_INACTIVE');
            }
            this.isValidUser(operador);
            const { dataset: { serverUrl: server } } = await License.getServerByOrganization({ NRORG });
            const { SERVER_DNS = '', NMORGANIZACAO = '' } = server;
            let token = '';
            const ACCESS_PROFILE = { PERMISSIONS: '', error: false, errorMessage: '' };
            const serverInfo = await DockerAssistant.handleClientServer(NRORG, SERVER_DNS).catch((e) => {
                defaultErrorHandler(e, 'ERR_CREATE_CONTAINER_ORG');
            });
            if (!SERVER_DNS && serverInfo.serverName) {
                await this.setServerForOrganization({
                    NRORG,
                    HOST: serverInfo.host,
                });
            }
            if (agent === 'assistant-installer') {
                AssistantInstaller.validateAgentVersion(agentVersion);
            }
            else {
                await this.testDbConnection(SERVER_DNS || serverInfo.host, NRORG).catch((e) => {
                    defaultErrorHandler(e, 'ERR_CONNECTION_DB');
                });
                token = await this.getToken(SERVER_DNS || serverInfo.host, { CDOPERADOR, NMOPERADOR, NRORG }).catch((e) => {
                    defaultErrorHandler(e, 'ERR_GENERATE_TOKEN');
                });
                await this.getAccessProfile(SERVER_DNS || serverInfo.host, { CDOPERADOR }).then((accessProfile) => {
                    Object.assign(ACCESS_PROFILE, accessProfile);
                }).catch((e) => {
                    ACCESS_PROFILE.error = true;
                    ACCESS_PROFILE.errorMessage = e.message;
                });
            }
            return {
                STATUS: true,
                STATUS_MSG: 'SUCCESS',
                STATUS_DATA: {},
                NODE: SERVER_DNS || serverInfo.host,
                NRORG: NRORG.toString(),
                NMORGANIZACAO,
                TOKEN: token,
                CDOPERADOR,
                NMOPERADOR,
                ACCESS_PROFILE,
            };
        }
        catch (error) {
            return {
                STATUS: false,
                STATUS_MSG: error.message,
                STATUS_DATA: errorInfo,
            };
        }
    }
}
const instance = new Login();
export { instance as Login };
export default instance;
